#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCHR 1
#define VENDOR_RQ_GET_SWITCHO 2
#define VENDOR_RQ_GET_SWITCHG 3
#define VENDOR_RQ_GET_SWITCHB 4

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return (PINC >>pin) & 1;
} 

void out_c(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */
	int i=0,n;
    PORTC  = 0b00100000;	
    DDRC  &= 0b11001010;	
    DDRC  |= 0b00001010;	
    PORTC &= 0b11110101;	
    PORTC |= 0b00110101; 

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
//        uint8_t led_val = rq->wValue.bytes[1];
//        out_c(led_no, led_val);
		if(led_no == 2) //Blue Led
			PORTC |= 0b00001010;
//			PORTC = 0b00001010;
		else if(led_no == 1){ //Green Led
			PORTC &= 0b11110111;	    
    	    PORTC |= 0b00000010;
//    	    PORTC = 0b00000010;
		}
		else if(led_no == 3){ //RED Led
			PORTC &= 0b11110101;
		}
		else{ //Orange Led
			PORTC &= 0b11111110;	    
		    PORTC |= 0b00001000;
//		    PORTC = 0b00001000;
		}
        return 0;
    }

    else if (rq->bRequest == VENDOR_RQ_GET_SWITCHR){
        dataBuffer[0] = !in_c(5);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCHO){
        dataBuffer[0] = !in_c(4);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCHG){
        dataBuffer[0] = !in_c(2);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCHB)
    {
        dataBuffer[0] = !in_c(0);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
      PORTC  = 0b00100000;	
      DDRC  &= 0b11001010;	
      DDRC  |= 0b00001010;	
      PORTC &= 0b11110101;	
      PORTC |= 0b00110101; 

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
